/**
 * Support code for inserting form/field objects into tinymce
 * Modified copy of code from ...\tiny_mce\plugins\table\js\table.js
 *
 * All tinymce functions are called from within this file.
 *
 * Form Wizard specific code in:
 * ...\tiny_mce\plugins\revize_form_wizard\js\formwizard.js
 */

//if (tinyMCEPopup.requireLangPack)	//if not called as standalone for html/css design
//	tinyMCEPopup.requireLangPack();

/*
 * This hack updates title for inline popup windows
 * (used firebug inspect element to find id pattern)
 */
function updateTitle(title)
{
	window.document.title = title;
	var tags = parent.document.getElementsByTagName('span')
	for (var i = 0; i < tags.length; i++)
		if (tags[i].id && tags[i].id.search(/mce_.*_title/) == 0)
			tags[i].innerHTML = title;
}

function closePopup()
{
	tinyMCEPopup.close();
}

function findInputType(type)
{
	var tags = FW.dom.doc.getElementsByTagName('input')
	if (tags)
	{
		for (var i=0;i<tags.length;i++)
			if (tags[i].type == type) return true;
	}
	return false;
}

/**
 * Insert or update object specified by type
 * cloned from ...\tiny_mce\plugins\table\js\table.js::insertTable()
 * @param {string} type: FWform, FWtextarea, FWbutton, etc.
 */
function insertHTML(fieldType)
{
	var inst = tinyMCEPopup.editor;
	var html = '';
	var elm = FW.focusElm;
	var attributes = FW.fieldAttributes;
	var tagName = FW.tagName;
	var update = FW.update;
	if (fieldType == 'FWInspection')
	{
		tagName = 'FWInspection';
	}

	if (fieldType == 'FWGeneralRequest')
	{
		tagName = 'FWGeneralRequest';
	}

	if (fieldType == 'FWform')
	{
		elm = FW.formElm;
		attributes = FW.formAttributes;
		tagName = 'form';
		update = true;
		if (!elm) //form does not exist
		{
			update = false;
			attributes.onsubmit = "return validate();";
			attributes.method = "post";
			attributes['accept-charset'] = "ISO-8859-1";
			// on publish, action filename is changed to:
			//	<published page name>_1.<technology>
			// name of tinymce lets us know how form was created
			attributes.action = "tinymce." + FW.rzform.technology.toLowerCase();
		}
		// force enctype; also forced during RZsave() so this is mostly cosmetic for html view
		attributes.encType = "multipart/form-data";
		if (FW.rzform.savemode == 'ftp')
			attributes.encType = "application/x-www-form-urlencoded";
		//else
			//attributes.encType = "text/plain";

		attributes.id = "formwizziwmorf";
		attributes.name = "formwizard";
	}
	else if (fieldType == 'FWCFB')
	{
		update = false;		//always replacing for quick builder
	}
	else if (fieldType == 'FWInspection')
	{
			update = false;		//always replacing for quick builder
	}
	else if (fieldType == 'FWGeneralRequest')
	{
			update = false;		//always replacing for quick builder
	}
	else
	{
		if (!attributes.id) attributes.id = attributes.name
		if (!attributes.previousname) attributes.previousname = attributes.name
	}

	var html = tagName.toLowerCase();
	var selectOptions = '';

	// same tag if not new element and same tagName
	var isSameTag = (elm && elm.tagName.toLowerCase() == tagName);

	// if not same type (for textfields), then tag must be inserted or replaced
	if (fieldType != 'FWform'
	&& FW.fieldAttributes.type && FW.fieldAttributes.type != elm.type)
		isSameTag = false;

	if (update)
	{
		// setAttrib does not clear checked attribute so tag must be replaced
		if ((fieldType == 'FWradio' || fieldType == 'FWcheckbox')
		&& elm.attributes.checked && !FW.fieldAttributes.checked)
			isSameTag = false;

		// firefox can not update textarea values
		if (fieldType == 'FWtextfield'
		&& elm.attributes.value != FW.fieldAttributes.value)
			isSameTag = false;

		// retain prior attributes if new value is not blank
		if (!isSameTag)
		{
			for (attr in FW.fieldAttributes)
			{
				if (FW.fieldAttributes[attr] != '')
					setAttrib(attr,FW.fieldAttributes);
			}
		}
	}

	// are both form and field tags being inserted
	var isFormAndField = (!FW.formElm
	                   && FW.formWindow.search(new RegExp(fieldType,'i')) == -1)


	if (tagName == 'input' && (FW.fieldAttributes.type == 'Submit' || FW.fieldAttributes.type == 'Reset')){
		//For submit and reset buttons do not add id and name
		setAttrib(['required','previousname'])
	} else {
		// all tags
		setAttrib(['id','name','required','previousname'])
	}
	switch (tagName)
	{
		case 'form':
			setAttrib('onsubmit,method,encType,accept-charset,action')
			html += '>'
			if (!isFormAndField)
				html += '<br /><br />'
			html += '</form';
			break;

		case 'input':	//includes FWbutton
			if (attributes.texttype)
				setAttrib('texttype');
			if (!attributes.maxlength
			|| (!isNaN(attributes.maxlength) && attributes.maxlength > 10000))
			{	//tinymce insists on adding attribute if not defined
				attributes.maxlength = "0";
			}
			setAttrib('type,value,maxlength');
			
			if (FW.fieldAttributes.type == 'radio'
			|| FW.fieldAttributes.type == 'checkbox')
				setAttrib('checked');
			
			else if (FW.fieldAttributes.type == 'file')
				setAttrib('multiple');
			
			else
				setAttrib('size');
			
			break;

		case 'textarea':
			setAttrib('texttype,rows,cols,maxlength,wrap,style,value');
			html += '>' + FW.fieldAttributes.value + '</textarea'
			break;

		case 'select':
			setAttrib('multiple,size');

			// if existing select element, replace options
			if (elm && isSameTag)
			{
				// clear existing options
				var opt;
				if (elm.options)
				{
					for(opt=elm.options.length-1;opt>-1;opt--)
						elm.options[opt] = null;
					elm.options.length = 0;
					elm.selectedIndex = -1;
				}

				// add options from dialog page
				for (opt=0;opt<FW.fieldAttributes.options.length;opt++)
				{
					var selected = (opt==FW.fieldAttributes.selectedIndex);
					elm.options[opt] = new Option(FW.fieldAttributes.options[opt].text,
											 	  FW.fieldAttributes.options[opt].value,
											 	  selected,selected);
				}
				elm.selectedIndex = FW.fieldAttributes.selectedIndex;
			}

			// build html for insert or replace
			html += '>' + FW.fieldAttributes.optionsHTML + '</select'
			break;

		case 'FWInspection':
			html = FWInspection_html('FWInspection');
			break;

		case 'FWGeneralRequest':
			html = FWGeneralRequest_html('FWGeneralRequest');
			break;

		default:	//form builder
			html = FWquickbuilder_html('FWCFB');
	}

	if (tagName && html && html.substring(0,1) != '<')
		html = '<' + html;
	if (tagName && html && html.lastIndexOf('</') == -1)
		html += ' /';
	if (tagName && html && html.substring(html.length-1) != '>')
		html += '>';

	// if changing existing node
	if (update && isSameTag)
	{
		inst.nodeChanged();
	}
	//if inserting form tag and field will follow, save form html until next call
	else if (tagName == 'form' && isFormAndField)
	{
		FW.html = html;
		return;
	}
	else
	{
		tinyMCEPopup.restoreSelection();
		if (FW.html) 											//if inserting form...
			html = FW.html.replace(/(>)(<)/,'$1'+html+'$2');	//...wrap field with form

		if (update)
			inst.execCommand('mceReplaceContent', false, html);
		else
		{
			inst.execCommand('mceInsertContent', false, html);
			if (fieldType == 'FWCFB')
				inst.addVisual(inst.dom.doc.body);	//table visual aids
		}
	}

	//TODO: select inserted or replaced element
	//inst.execCommand('mceSelectNode',false,attributes.id)

	FW.isForm = true;	//indicates form exists
	return;
	/**
	 * Internal function to update an array of attribute(s)
	 * (alternatively attributes can be passed as a csv)
	 * @param {array} attrArray list of attributes
	 */
	function setAttrib(attrArray)
	{
		if (typeof attrArray == 'string')
			attrArray = attrArray.split(',');
		for (var i = 0; i < attrArray.length; i++)
		{
			var attrib = attrArray[i];
			var value = attributes[attrib];
			if (typeof value != 'undefined' && typeof value != 'object'
			&& (value != '' || FW.omittedAttributes[attrib]))
			{
				value = value.replace(/&/g, '&amp;');
				value = value.replace(/\"/g, '&quot;');
				value = value.replace(/</g, '&lt;');
				value = value.replace(/>/g, '&gt;');
				if (update && isSameTag)
					FW.dom.setAttrib(elm, attrib, value);
				if (value)
					html += ' ' + attrib + '="' + value + '"';
			}
		}
	}
}
/*
<form id="formwizziwmorf" name="formwizard" onsubmit="return validate();" action="index.jsp" method="post" encType="text/plain" >
textarea:text
<textarea style="OVERFLOW: hidden" name="textfield" rows="1" wrap="default"
 maxlength="40" required="true" texttype="textfield" previousname="textfield"
 value="value">value</textarea><br />
textarea:date
<input maxLength="40" value="1/1/2000" name="dateField" texttype="date"
 previousname="dateField"/><br />
textarea:numeric value
<input maxLength="40" value="1" name="numberValue" texttype="numeric"
 previousname="numberValue"/><br />
textarea:decimal value
<input maxLength="40" value="true" name="decimalValue" required="true" texttype="decimal"
 previousname="decimalValue"/><br />
textarea:password
<input type="password" maxLength="40" value="password" name="passwordField" required="true"
 previousname="passwordField"/><br />
textarea:hidden
<input type="hidden" value="hiddenValue" name="hiddenfield" previousname="hiddenfield" /><br /></p>

radio
<input type="radio" value="yes" name="radioField" previousname="radioField"/><br />
checkbox
<input type="checkbox" CHECKED value="x" name="checkboxField" required="true" previousname="checkboxField"/><br /><select multiple size="3" name="listField" previousname="listField"><option value="opt1" >opt1</option><option value="opt2" >opt2</option></select>

<input id="s1" type="submit" value="submit" name="submit" previousname="submit"/>
<input id="r1" type="reset" value="reset" name="reset" previousname="reset"/>
*/

/**
 * Called from revize_form_wizard/editor_plugin::init()
 * cloned from ...\tiny_mce\plugins\table\js\table.js::init()
 */
function init()
{
	tinyMCEPopup.resizeToInnerSize();
	var inst = tinyMCEPopup.editor;

	FW = inst.FW;		//FW created during editor instanciation via callback
	FW.dom = inst.dom;
	FW.command = tinyMCEPopup.getWindowArg('command');
	FW.validFieldTags = {textarea:true,input:true,select:true};
	FW.omittedAttributes = {checked:true,multiple:true};

	// if html contained rz:form tag, the FW object is created during inst instanciation
	// create if not yet created (page does not contain form)
	if (!FW.rzform)
		FW.rzform = {}		//for <rz:form...> properties

	// determine implied property (used by validate)
	if (FW.rzform.email)
		FW.rzform.savemode = 'email';
	else if (FW.rzform.filename)
		FW.rzform.savemode = 'ftp';

	// save focusElm attributes in FW.attributes
	FW.fieldAttributes = {};
	FW.fieldAttributes.options = [];
	FW.focusElm = inst.selection.getNode();
	if (FW.focusElm.tagName && FW.validFieldTags[FW.focusElm.tagName.toLowerCase()])
	{
		saveAttributes(FW.focusElm,FW.fieldAttributes);
		if (FW.focusElm.tagName == 'SELECT' && FW.focusElm.innerHTML)
		{
			for (var i=0;i<FW.focusElm.childNodes.length;i++)
			{
				var node = FW.focusElm.childNodes[i];
				if (node.tagName == 'OPTION'
				&& (node.value || node.text))
				{
					var opt = FW.fieldAttributes.options.length
					FW.fieldAttributes.options[opt] = {};
					FW.fieldAttributes.options[opt].text = node.text;
					FW.fieldAttributes.options[opt].value = node.value;
					if (node.selected)
						FW.fieldAttributes.selectedIndex = opt;
				}
			}
		}
	}

	// save form attributes in FW.form
	FW.formAttributes = {};
	FW.formElm = inst.dom.getParent(FW.focusElm, "form");
	saveAttributes(FW.formElm,FW.formAttributes);

	FWsetup(inst);	//finish loading (select form for right pane)

	/**
	 * Internal function to save (specified) attributes of tag into obj
	 * @param {Object} element to get attributes to save in obj
	 * @param {Object} obj (e.g. FW.fieldAttrributes or FW.formAttributes)
	 */
	function saveAttributes(elm,obj)
	{
		if (!elm) return;	//no elm specified

		for (var i=0;i<elm.attributes.length;i++)
		{
			if (elm.attributes[i].specified && elm.attributes[i].name
			&& elm.attributes[i].value && elm.attributes[i].value != 'null')
				obj[elm.attributes[i].name.toLowerCase()] = elm.attributes[i].value;
		}
		if (elm.value)
			obj.value = elm.value;
		if (obj.style
		&& obj.mce_style
		&& obj.mce_style.toLowerCase().indexOf('overflow') == 0)
		{
			obj.style = obj.mce_style;
			obj.mce_style = ''
		}
	}
}

// schedule above init() unless containing html page called as standalone for testing
if (tinyMCEPopup.onInit)
	tinyMCEPopup.onInit.add(init);
