


tinymce.PluginManager.add('revize_form_wizard', function(editor, url) {
	
	var formEditor,
	fBuilder,
	fData,
	formOptions = {
		stickyControls: {
			enable: true
		},
		disableHTMLLabels: true,
		defaultFields: [],
		disabledAttrs: ['access'],
		disableFields: ['autocomplete','button','hidden'],
		controlOrder: ['header','paragraph','text','textarea'],
		disabledActionButtons: ['data','save'],
		disabledSubtypes: {
		  textarea: ['tinymce','quill']
		},
		inputSets:[
			{
				label: 'Citizens Request Center',
				name: 'citizens-request-center',
				icon: '✉',
				showHeader: true,
				fields: [
					{
						type: 'select',
						required: true,
						label: 'Department',
						description: 'Department that receives the issue/request',
						className: 'form-control',
						name: 'whichemailaddress',
						access: false,
						multiple: false,
						values: [
						{
							label: 'Department 1',
							value: 'department1@email.com',
							selected: true
						},
						{
							label: 'Department 2',
							value: 'department2@email.com'
						}
						]
					},{
						type: 'text',
						label: 'Your First Name',
						className: 'form-control',
						name: 'first_name',
						required: true
					},{
						type:'text',
						label: 'Your Last Name',
						className: 'form-control',
						required: true
					},{
						type: 'text',
						subtype: 'email',
						required: true,
						label: 'Email',
						placeholder: 'email@email.com',
						className: 'form-control',
						name: 'email',
						access: false,
						maxlength: 120
					},{
						type: 'text',
						subtype: 'tel',
						required: true,
						label: 'Phone',
						className: 'form-control',
						name: 'phone',
						access: false,
						maxlength: 15
					},{
						type: 'textarea',
						required: true,
						label: 'Issue',
						description: 'Please write your issue here',
						className: 'form-control',
						name: 'issue',
						access: false,
						subtype: 'textarea',
						rows: 4
					},{
						type: 'radio-group',
						required: true,
						label: 'Preferred Method of Contact',
						description: 'How would you like us to contact you?',
						inline: true,
						name: 'preferred_contact',
						access: false,
						other: false,
						values: [
							{
								label: 'Email',
								value: 'email',
								selected: true
							},
							{
								label: 'Phone',
								value: 'phone'
							}
						]
					}
				]
			}
		]
	};

	// Generate and return Form Properties
	var getFormProperties = function(){
		var fName = "", fSubject = "", fEmail = "", fSuccess = "", fFail = "", fCC = "";

		if(typeof editor.FW.rzform !== 'undefined'){
			fName = typeof editor.FW.rzform.formName !== 'undefined' ? editor.FW.rzform.formName : "";
			fSubject = typeof editor.FW.rzform.subject !== 'undefined' ? editor.FW.rzform.subject : "";
			fEmail = typeof editor.FW.rzform.email !== 'undefined' ? editor.FW.rzform.email : "";
			//fFrom = typeof editor.FW.rzform.from !== 'undefined' ? editor.FW.rzform.from : "";
			fCC = typeof editor.FW.rzform.cc !== 'undefined' ? editor.FW.rzform.cc : "";
			fSuccess = typeof editor.FW.rzform.successpage !== 'undefined' ? editor.FW.rzform.successpage : "";
			fFail = typeof editor.FW.rzform.errorpage !== 'undefined' ? editor.FW.rzform.errorpage : "";
		}

		return('<div class="rzform-props">'
		+	'<h3>Form Properties</h3>'
		+	'<div class="form-group"><label for="rzFormName">Form Name</label><input type="text" class="form-control" name="rzFormName" id="rzFormName" value="'+fName+'"></div>'
		+	'<div class="form-group"><label for="rzFormSubject">Subject</label><input type="text" class="form-control" name="rzFormSubject" id="rzFormSubject" value="'+fSubject+'"></div>'
		+	'<div class="form-group"><label for="rzFormEmail">Email</label><input type="email" class="form-control" name="rzFormEmail" id="rzFormEmail" value="'+fEmail+'"></div>'
		+	'<div class="form-group"><label for="rzFormCC">CC</label><input type="text" class="form-control" name="rzFormCC" id="rzFormCC" value="'+fCC+'"></div>'
		+	'<div class="form-group"><label for="rzFormSuccess">Sucess URL <small>(Optional)</small></label><input type="text" class="form-control" name="rzFormSuccess" id="rzFormSuccess" value="'+fSuccess+'"></div>'
		+	'<div class="form-group"><label for="rzFormFailure">Failure URL <small>(Optional)</small></label><input type="email" class="form-control" name="rzFormFailure" id="rzFormFailure" value="'+fFail+'"></div>'
		+	'</div>');
	}


	var openDialog = function () {
	  return editor.windowManager.open({
		title: 'Revize Form Builder',
		size: 'large',
		body: {
		  type: 'panel',
		  items: [
			// {
			//   type: 'input',
			//   name: 'title',
			//   label: 'Revize Form Builder'
			// }
			{
				type: 'htmlpanel', // A HTML panel component
				html: '<div id="dialog-build-wrap" class="build-wrap"></div>'
			}
		  ]
		},
		buttons: [
		  {
			type: 'cancel',
			text: 'Close'
		  },
		  {
			type: 'submit',
			text: 'Save',
			primary: true
		  }
		],
		onSubmit: function (api) {

			// Get the Form Data
			fData = JSON.parse(fBuilder.actions.getData('json'));

			var validateForm = function(str){
				var emailRegex = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
				if(str.indexOf('rendered-form') >= 0){
					if( $('#rzFormEmail').val() === ""
					||	$('#rzFormName').val() === ""
					||	$('#rzFormSubject').val() === ""){
						tinymce.activeEditor.windowManager.alert('Form properties need to be entered first. Please click the "Form Properties" button to fill out the required values.');
						return false;
					} else if($('#rzFormEmail').val() !== "" && !emailRegex.test($('#rzFormEmail').val())){
						tinymce.activeEditor.windowManager.alert('Email entered under Form Properties is invalid. Please check for a valid email. Contact Revize Support if the email entered is valid');
						return false
					} else if($('#rzFormCC').val() !== "" && !emailRegex.test($('#rzFormCC').val())) {
						tinymce.activeEditor.windowManager.alert('CC Email entered under Form Properties is invalid. Please check for a valid email. Contact Revize Support if the email entered is valid');
						return false;
					} else {
						return true;
					}
				}
			}

			// Create the html in the dop
			var $tempForm = formJQuery('<div />');
			var formId = "formwizziwmorf";
			var $editingHTML = $('<div/>').html($(editor.getContent()));
			if(!parent.window.isEditingForm){
				parent.window.formCount++;
			}

			// Remove any attributes if they equal false
			for(var i = 0; i < fData.length; i++){
				for(var p in fData[i]){
					if(!fData[i][p]) delete fData[i][p];
				}
			}

			// Render using scrubbed data
			$tempForm.formRender({
				formData: fData
			});

			// Markup String
			var markupStr = $tempForm.formRender('html');

			// Validate Form Fields
			if(!validateForm(markupStr)){
				return false;
			}
			

			// Insert form into editor and close
			if(!parent.window.isEditingForm){
				editor.execCommand('mceInsertContent', false, '<br>'
				+ '<form id="'+formId+'" style="position:relative" class="formwiznew" name="formwizard" action="tinymce.jsp" accept-charset="ISO-8859-1" enctype="multipart/form-data" method="post">' 
				+ '<div style="display:none" class="tiny-form-editbtn" data-formData="' + JSON.stringify(fData).replace(/\"+/g,"'") + '">Edit Form</div>'
				+ markupStr 
				+ '<input type="submit" value="Submit"></form><br>&nbsp;');
			} else {
				if(fData.length){
					$($editingHTML).find('#'+formId).html('<div style="display:none" class="tiny-form-editbtn" data-formData="' + JSON.stringify(fData).replace(/\"+/g,"'") + '">Edit Form</div>'
					+ markupStr 
					+ '<input type="submit" value="Submit">');
				} else {
					$($editingHTML).find('#'+formId).remove();
					parent.window.formCount = 0;
				}
				editor.setContent($($editingHTML).html());
			}
			
			
			editor.FW.isForm = true;
			editor.FW.rzform.formId = RZ.pageid;
			editor.FW.rzform.errorpage = "";
			editor.FW.rzform.successpage = "";
			editor.FW.rzform.cc = $('#rzFormCC').val();
			editor.FW.rzform.email = $('#rzFormEmail').val();
			editor.FW.rzform.formName = $('#rzFormName').val();
			editor.FW.rzform.email_password = "xxx";
			editor.FW.rzform.email_user = "xxx";
			editor.FW.rzform.errorpage = $('#rzFormFailure').val();
			editor.FW.rzform.from = "noreply@revize.com";
			editor.FW.rzform.id = "rzformmrofzr";
			editor.FW.rzform.password = "gorevize";
			editor.FW.rzform.savemode = "email";
			editor.FW.rzform.smtp_server = "xxx";
			editor.FW.rzform.smtpserver = "mail.revize.com";
			editor.FW.rzform.subject = $('#rzFormSubject').val();
			editor.FW.rzform.successpage = $('#rzFormSuccess').val();
			editor.FW.rzform.technology = "jsp";
			editor.FW.rzform.user = "workflowuser@revize.com";
			editor.FW.formAttributes = {};
			editor.FW.formAttributes['accept-charset'] = "ISO-8859-1";
			editor.FW.formAttributes.action = "tinymce.jsp";
			editor.FW.formAttributes.encType = "multipart/form-data";
			editor.FW.formAttributes.enctype = "text/plain";
			editor.FW.formAttributes.id = formId;
			editor.FW.formAttributes.method = "post";
			editor.FW.formAttributes.name = $('#rzFormName').val();
			editor.FW.formAttributes.onsubmit = "return validate();";
			var node = $(editor.selection.getNode());
			editor.FW.formElm = $(node).find('#'+formId)[0];
			
			parent.window.isEditingForm = false;
			parent.window.formData = [];
    		api.close();
		}
	  });
	};
	
	editor.ui.registry.addIcon('form-builder', '<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" width="24" height="22" viewBox="0 0 24 18"><image id="Rectangle_4" data-name="Rectangle 4" width="24" height="18" xlink:href="data:img/png;base64,iVBORw0KGgoAAAANSUhEUgAAABgAAAAWCAYAAADafVyIAAAAYklEQVRIiWNU0rf7z0BDwERLw0GABYnNSGWzwSFDVx/QJC5o7gPG//9pmojo4ANK88HdCwfxpr5BnQ+I8vloPiAIaJ4PQMmUaMV3Lxwk2QIWEsKerNKW5nEwasEAW8DAwAAAOb8TssJusJYAAAAASUVORK5CYII="></image></svg>');

	// Add a button that opens a window
	editor.ui.registry.addButton('formwizard', {
	  icon: 'form-builder',
	  onAction: function () {
		  
		
		  if(parent.window.formCount > 0){
			var formDataStr = "";
			var $editorHTML = $('<div/>').html($(editor.getContent()));

			if($($editorHTML).find('div.tiny-form-editbtn').length){
				formDataStr = $($editorHTML).find('div.tiny-form-editbtn').attr('data-formdata');
				window.formData = formDataStr !== '' ? JSON.parse(formDataStr.replace(/\'/g,'"')) : [];
				window.isEditingForm = true;
				window.currentFormId = 'formwizziwmorf';
				
				// Open window
				openDialog();
				formOptions.defaultFields = window.formData;
				formEditor = document.getElementById('dialog-build-wrap');
				fBuilder = formJQuery(formEditor).formBuilder(formOptions).promise.then(function(fb){
					$('ul.frmb-control').before(getFormProperties());
					$('.rzform-props>h3').on('click',function(){
						$('.rzform-props .form-group').slideToggle();
					});
					fBuilder = fb;
				});
			} else {
				alert('Pages may only have 1 Revize Form. Please remove the form another editor first.');
			}

			window.formData = formDataStr !== '' ? JSON.parse(formDataStr.replace(/\'/g,'"')) : [];
			window.isEditingForm = true;
		  } else {
			// Open window
			openDialog();
			formOptions.defaultFields = window.formData;
			formEditor = document.getElementById('dialog-build-wrap');
			fBuilder = formJQuery(formEditor).formBuilder(formOptions).promise.then(function(fb){
				$('ul.frmb-control').before(getFormProperties());
				$('.rzform-props>h3').on('click',function(){
					$('.rzform-props .form-group').slideToggle();
				});
				fBuilder = fb;
			});
		  }// end else
	  }
	});
  
	// Adds a menu item, which can then be included in any menu via the menu/menubar configuration
	// editor.ui.registry.addMenuItem('formwizard', {
	//   icon: 'comment-add',
	//   onAction: function() {
		 
	// 	// Open window
	// 	openDialog();
	// 	formOptions.defaultFields = window.formData;
	// 	formEditor = document.getElementById('dialog-build-wrap');
	// 	fBuilder = $(formEditor).formBuilder(formOptions);
	//   }
	// });
  
	return {
	  getMetadata: function () {
		return  {
		  name: "revize_form_wizard",
		  url: "http://revize_form_wizardplugindocsurl.com"
		};
	  }
	};
  });
